/*
 * Project: Examples for GJA course
 * Author: Ing. Jaroslav Dytrych idytrych@fit.vutbr.cz
 * File: FileUploadServlet.java
 * Description: Example servlet for file upload
 */

/**
 * @file FileUploadServlet.java
 *
 * @brief Example servlet for file upload
 */

/**
 * @package cz.vutbr.fit.knot.gja.fileUpload
 * 
 * @brief Example of file upload
 */
package cz.vutbr.fit.knot.gja.fileUpload;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Iterator;
import java.util.List;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;

/**
 * Example servlet for file upload
 * 
 * @author ikouril
 * @author idytrych
 */
@WebServlet(name = "FileUpload", urlPatterns = {"/FileUpload"})
public class FileUploadServlet extends HttpServlet {

  /**
   * The serialization runtime associates with each serializable class 
   * a version number, called a serialVersionUID, which is used during 
   * deserialization to verify that the sender and receiver of a serialized 
   * object have loaded classes for that object that are compatible with 
   * respect to serialization.
   * https://docs.oracle.com/javase/7/docs/api/java/io/Serializable.html
   */
  private static final long serialVersionUID = 4266718586952483675L;
  
  /**
   * Temporary directory for storing of uploaded files
   */
  private static final String TMP_DIR_PATH = "/tmp";
  /**
   * Temporary file for upload
   */
  private File tmpDir;
  /**
   * Destination directory for uploaded file
   */
  private static final String DESTINATION_DIR_PATH = ".";
  /**
   * An abstract representation of destination directory pathname
   */
  private File destinationDir;

  /**
   * Servlet initialisation
   * 
   * @param config Configuration object
   */
  public void init(ServletConfig config) throws ServletException {
    super.init(config);
    tmpDir = new File(TMP_DIR_PATH);
    if (!tmpDir.isDirectory()) {
      throw new ServletException(TMP_DIR_PATH + " is not a directory");
    }
    String realPath = getServletContext().getRealPath(DESTINATION_DIR_PATH);
    destinationDir = new File(realPath);
    if (!destinationDir.isDirectory()) {
      throw new ServletException(DESTINATION_DIR_PATH + " is not a directory");
    }

  }

  /**
   * Processes GET or POST request
   * 
   * @param request servlet request
   * @param response servlet response
   * @throws java.io.IOException if an I/O error occurs
   */
  protected void processRequest(HttpServletRequest request, HttpServletResponse response)
                                throws IOException {
    response.setContentType("text/html;charset=UTF-8");
    PrintWriter out = response.getWriter();
    response.setContentType("text/plain");
    out.println("Servlet File Upload Example using Commons File Upload");
    out.println();

    DiskFileItemFactory fileItemFactory = new DiskFileItemFactory();
    /*
     * Set the size threshold, above which content will be stored on disk.
     */
    fileItemFactory.setSizeThreshold(1 * 1024 * 1024); // 1 MB
    /*
     * Set the temporary directory to store the uploaded files of size above
     * threshold.
     */
    fileItemFactory.setRepository(tmpDir);

    ServletFileUpload uploadHandler = new ServletFileUpload(fileItemFactory);
    try {
      /*
       * Parse the request
       */
      List<?> items = uploadHandler.parseRequest(request);
      Iterator<?> itr = items.iterator();
      while (itr.hasNext()) {
        FileItem item = (FileItem) itr.next();
        /*
         * Handle Form Fields.
         */
        if (item.isFormField()) {
          out.println("File Name = " + item.getFieldName() + ", Value = " + item.getString());
        } else {
          // Handle Uploaded files.
          out.println("Field Name = " + item.getFieldName()
                  + ", File Name = " + item.getName()
                  + ", Content type = " + item.getContentType()
                  + ", File Size = " + item.getSize());
          /*
           * Write file to the ultimate location.
           */
          File file = new File(destinationDir, item.getName());
          item.write(file);
        }
        out.close();
      }
    } catch (FileUploadException ex) {
      log("Error encountered while parsing the request", ex);
    } catch (Exception ex) {
      log("Error encountered while uploading file", ex);
    }


  }  // processRequest()

  // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
  /**
   * Handles the HTTP
   * <code>GET</code> method.
   *
   * @param request servlet request
   * @param response servlet response
   * @throws IOException if an I/O error occurs
   */
  @Override
  protected void doGet(HttpServletRequest request, HttpServletResponse response)
          throws IOException {
    processRequest(request, response);
  }

  /**
   * Handles the HTTP
   * <code>POST</code> method.
   *
   * @param request servlet request
   * @param response servlet response
   * @throws IOException if an I/O error occurs
   */
  @Override
  protected void doPost(HttpServletRequest request, HttpServletResponse response)
          throws IOException {
    processRequest(request, response);
  }

  /**
   * Returns a short description of the servlet.
   *
   * @return a String containing servlet description
   */
  @Override
  public String getServletInfo() {
    return "File upload servlet";
  }// </editor-fold>
}  // public class FileUploadServlet
